/*
 * EDSHOT Change Log
 *
 * rev1
 * -Translations
 * -Switched to SourceMod voting API
 *
 * rev2
 * -Fixed a bug where switching to a map doesn't set the correct gamemode automatically.
 *  This also means you need to modify your matchmap.txt to update the 1st section names (coop/survival).
 *
 *
 * EDSHOT Bug List
 * -It may be possible to do the 'map transition pass' bug here as well. Haven't tried though.
 * -There are times when a tied vote (1-1) passes anyways. I don't see any problems here so I am just going to
 *  assume its another SourceMod voting API bug.
 */

#include <sourcemod>
#include <sdktools>
#include "edlib.sp"

#define MATCHMODES_PATH		"configs/matchmap.txt"
#define MATCHMODES_PREFIX	"[matchmapvote] "

#define VOTE_SOUND_START	"ui/beep_synthtone01.wav"
#define VOTE_SOUND_PASS		"ui/menu_enter05.wav"
#define VOTE_SOUND_FAIL		"ui/beep_error01.wav"
#define VOTE_TIMEOUT		20

new Handle:g_hModesKV 			= INVALID_HANDLE;
new Handle:g_hCvarChangeMapWaitTime 	= INVALID_HANDLE;
new String:g_sMap[64];
new String:g_sGameMode[64];
new String:tBuffer[256];
bool IsMatchVoteInProgress		= false;

public Plugin:myinfo = 
{
	name = "Match Map Vote",
	author = "vintik, epilimic, EDSHOT",
	description = "type !matchmap to vote a new custom map",
	version = "1.3-rev2"
}

public void OnPluginStart()
{
	new String:sBuffer[128];

	GetGameFolderName(sBuffer, sizeof(sBuffer));
	if (!StrEqual(sBuffer, "left4dead", false) && !StrEqual(sBuffer, "left4dead2", false))
	{
		SetFailState("This plugin is only supported by the Left 4 Dead series.");
	}

	g_hModesKV = CreateKeyValues("MatchModes");
	BuildPath(Path_SM, sBuffer, sizeof(sBuffer), MATCHMODES_PATH);
	if (!FileToKeyValues(g_hModesKV, sBuffer))
	{
		SetFailState("Couldn't load configs/matchmap.txt!");
	}

	RegConsoleCmd("sm_matchmap", MatchRequest);
	RegConsoleCmd("sm_matchmapvote", VoteRedraw);
	g_hCvarChangeMapWaitTime = CreateConVar("sm_matchmap_waittime", "10", "Number of seconds before map change", FCVAR_PLUGIN, true, 10.0, false, 1.0);

	PrecacheSound(VOTE_SOUND_START, true);
	PrecacheSound(VOTE_SOUND_PASS, true);
	PrecacheSound(VOTE_SOUND_FAIL, true);

	LoadTranslations("common.phrases");
	LoadTranslations("matchmapvote.phrases");
}

public void OnConfigsExecuted()
{
	PrecacheSound(VOTE_SOUND_START, true);
	PrecacheSound(VOTE_SOUND_PASS, true);
	PrecacheSound(VOTE_SOUND_FAIL, true);

	// 
	// In case someone called the vote right before the map transition.
	// - EDSHOT
	// 
	IsMatchVoteInProgress = false;
}

public Action:MatchRequest(client, args)
{
	if (client != 0 && IsClientInGame(client)) {}
	else { return Plugin_Handled; }

	if (args > 0)
	{
		//config specified
		new String:sMap[64];
		new String:sBuffer[128];
		GetCmdArg(1, sMap, sizeof(sMap));
		
		if (IsMapValid(sMap))
		{
			FindMapName(sMap, sBuffer, sizeof(sBuffer));
			if (StartMatchVote(client, sBuffer))
			{
				strcopy(g_sMap, sizeof(g_sMap), sMap);
				PrintToChatAllTranslated1I1C("PlayerStartedVote", MATCHMODES_PREFIX, "", client, g_sGameMode);
				PrintToChatAll(sBuffer);
				//PrintToChatAllTranslated("MatchVoteCommandHint", MATCHMODES_PREFIX, ".");
				PrintToChatAllTranslated("MatchVoteMapChangeCaution", MATCHMODES_PREFIX, "!");
			}
			return Plugin_Handled;
		}
		else
		{
			Format(tBuffer, sizeof(tBuffer), "%s%T!", MATCHMODES_PREFIX, "MapDoesNotExist", client, sBuffer);
			PrintToChat(client, tBuffer);
			return Plugin_Handled;
		}
	}
	else
	{
		//show main menu
		MatchModeMenu(client);
	}

	return Plugin_Handled;
}

public bool FindMapName(const String:cfg[], String:name[], int maxlength)
{
	KvRewind(g_hModesKV);
	if (KvGotoFirstSubKey(g_hModesKV))
	{
		do
		{
			if (KvJumpToKey(g_hModesKV, cfg))
			{
				KvGetString(g_hModesKV, "name", name, maxlength);
				return true;
			}
		} while (KvGotoNextKey(g_hModesKV));
	}
	return false;
}

public Action:MatchModeMenu(client)
{
	new Handle:hMenu = CreateMenu(MatchModeMenuHandler);
	Format(tBuffer, sizeof(tBuffer), "%T:", "MatchModeMenuTitle", client);
	SetMenuTitle(hMenu, tBuffer);
	new String:sBuffer[128];
	KvRewind(g_hModesKV);
	if (KvGotoFirstSubKey(g_hModesKV))
	{
		do
		{
			KvGetSectionName(g_hModesKV, sBuffer, sizeof(sBuffer));
			AddMenuItem(hMenu, sBuffer, sBuffer);
		} while (KvGotoNextKey(g_hModesKV));
	}
	DisplayMenu(hMenu, client, 20);
}

public MatchModeMenuHandler(Handle:menu, MenuAction:action, param1, param2)
{
	if (action == MenuAction_Select)
	{
		new String:sInfo[128];
		new String:sBuffer[128];
		GetMenuItem(menu, param2, sInfo, sizeof(sInfo));
		strcopy(g_sGameMode, sizeof(g_sGameMode), sInfo);
		KvRewind(g_hModesKV);
		if (KvJumpToKey(g_hModesKV, sInfo) && KvGotoFirstSubKey(g_hModesKV))
		{
			new Handle:hMenu = CreateMenu(ConfigsMenuHandler);
			SetMenuTitle(hMenu, sInfo);
			do
			{
				KvGetSectionName(g_hModesKV, sInfo, sizeof(sInfo));
				KvGetString(g_hModesKV, "name", sBuffer, sizeof(sBuffer));
				AddMenuItem(hMenu, sInfo, sBuffer);
			} while (KvGotoNextKey(g_hModesKV));
			DisplayMenu(hMenu, param1, 20);
		}
		else
		{
			Format(tBuffer, sizeof(tBuffer), "%T.", "NoConfigsFound", param1);
			PrintToChat(param1, tBuffer);
			MatchModeMenu(param1);
		}
	}
	if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
}

public ConfigsMenuHandler(Handle:menu, MenuAction:action, param1, param2)
{
	if (action == MenuAction_Select)
	{
		new String:sInfo[128];
		new String:sBuffer[128];
		GetMenuItem(menu, param2, sInfo, sizeof(sInfo), _, sBuffer, sizeof(sBuffer));
		if (StartMatchVote(param1, sBuffer))
		{
			strcopy(g_sMap, sizeof(g_sMap), sInfo);
			PrintToChatAllTranslated1I1C("PlayerStartedVote", MATCHMODES_PREFIX, "", param1, g_sGameMode);
			PrintToChatAll(sBuffer);
			//PrintToChatAllTranslated("MatchVoteCommandHint", MATCHMODES_PREFIX, ".");
			PrintToChatAllTranslated("MatchVoteMapChangeCaution", MATCHMODES_PREFIX, "!");
		}
		else
		{
			MatchModeMenu(param1);
		}
	}
	if (action == MenuAction_End)
	{
		CloseHandle(menu);
	}
	if (action == MenuAction_Cancel)
	{
		MatchModeMenu(param1);
	}
}

public bool StartMatchVote(client, const String:cfgname[])
{
	if (GetClientTeam(client) == 1)
	{
		Format(tBuffer, sizeof(tBuffer), "%s%T.", MATCHMODES_PREFIX, "SpectatorNotAllowedToVote", client);
		PrintToChat(client, tBuffer);
		return false;
	}
	// 
	// IsVoteInProgress() doesn't work.
	// 
	// IsVoteInProgress(Handle:menu) can't be done since the menu is created within this block.
	// Even then, the documentation says "Deprecated; no longer used."
	// - EDSHOT
	// 
	if (!IsMatchVoteInProgress)
	{
		new iNumPlayers;
		decl iPlayers[MaxClients];

		//list of non-spectators players
		for (int i = 1; i <= MaxClients; i++)
		{
			if (!IsClientInGame(i) || IsFakeClient(i) || (GetClientTeam(i) == 1))
			{
				continue;
			}
			iPlayers[iNumPlayers++] = i;
		}

		Menu menu = new Menu(VoteHandler);

		Format(tBuffer, sizeof(tBuffer), "%T", "ChangeTheModeQuestion", client, cfgname);
		menu.SetTitle(tBuffer);
		Format(tBuffer, sizeof(tBuffer), "%T", "Yes", client);
		menu.AddItem("", tBuffer);
		Format(tBuffer, sizeof(tBuffer), "%T", "No", client);
		menu.AddItem("", tBuffer);
		menu.ExitButton = false;

		EmitSoundToAll(VOTE_SOUND_START, -2, 0, 75, 0, 1.0, 100, -1, NULL_VECTOR, NULL_VECTOR, true, 0.0);
		menu.DisplayVote(iPlayers, MaxClients, VOTE_TIMEOUT);

		IsMatchVoteInProgress = true;
		return true;
	}
	Format(tBuffer, sizeof(tBuffer), "%s%T.", MATCHMODES_PREFIX, "MatchVoteCannotStart", client);
	PrintToChat(client, tBuffer);
	return false;
}

public Action:VoteRedraw(client, args)
{
	if (IsClientInVotePool(client))
	{
		RedrawClientVoteMenu(client, false);
	}
	else
	{
		Format(tBuffer, sizeof(tBuffer), "%s%T", MATCHMODES_PREFIX, "MatchVoteCommandAlreadyVoted", client);
		PrintToChat(client, tBuffer);
	}

	return Plugin_Handled;
}

public int VoteHandler(Menu menu, MenuAction action, int param1, int param2)
{
	switch (action)
	{
		case MenuAction_End:
		{
			IsMatchVoteInProgress = false;
		}
		case MenuAction_VoteEnd:
		{
			if (param1 == 0)
			{
				EmitSoundToAll(VOTE_SOUND_PASS, -2, 0, 75, 0, 1.0, 100, -1, NULL_VECTOR, NULL_VECTOR, true, 0.0);
				PrintToChatAllTranslated1I("ChangeModeAfterSeconds", MATCHMODES_PREFIX, ".", GetConVarInt(g_hCvarChangeMapWaitTime));
				//PrintToChatAllTranslated("MatchVoteMapChangeBlackScreen", MATCHMODES_PREFIX, ".");
				IsMatchVoteInProgress = false;
				CreateTimer((GetConVarInt(g_hCvarChangeMapWaitTime) * 1.0), COOL_DOWN);
			}
			else
			{
				EmitSoundToAll(VOTE_SOUND_FAIL, -2, 0, 75, 0, 1.0, 100, -1, NULL_VECTOR, NULL_VECTOR, true, 0.0);
				PrintToChatAllTranslated("VoteFailed", MATCHMODES_PREFIX, ".");
				IsMatchVoteInProgress = false;
			}
		}
	}
}

public Action:COOL_DOWN(Handle:timer)
{
	ServerCommand("sm_cvar mp_gamemode %s", g_sGameMode);
	ServerCommand("sm_map %s", g_sMap);
}
